<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for helping in work with headers
 */
class HeadersHelper
{
    /**
     * Build content headers for file content.
     * This is basically the same as buildStaticFileHeaders() except that the
     * path is relative to the contentSource / Repository directory.
     *
     * @deprecated
     * @see HeadersHelper::buildStaticFileHeaders()
     * @param string $path    file path, must be relative to the contentSource
     *                        cache directory
     * @param string $content Not used
     *
     * @return array headers
     */
    public static function forFile($path, $content = null) {
        return self::buildStaticFileHeaders(Yii::app()->getModule('contentSource')
                                ->getContentSource()->contentFile($path));
    }

    /**
     * Build content headers for file content.
     *
     * * Content-Type
     * * Content-Length (if $path actually exists)
     * * Last-Modified (if $path actually exists)
     *
     * @param type $path    Path to the file, the file must really exist
     * @return array HTTP headers
     */
    public static function buildStaticFileHeaders($path) {
        $headers = array();
        $headers[] = "Content-Type: " . CFileHelper::getMimeTypeByExtension($path);

        if (file_exists($path)) {
            $headers[] = "Content-Length: " . filesize($path);
            $headers[] = "Last-Modified: " . gmdate("D, d M Y H:i:s", filemtime($path)) . " GMT";
        }

        return $headers;
    }

    /**
     * Takes an array of HTTP headers and sends them to the
     * HTTP client. Already set HTTP headers will be overwritten.
     * @param array $headers HTTP headers as strings
     */
    public static function sendHttpHeaders($headers) {
        // Send each HTTP header with the header PHP function
        array_map("header", $headers);
    }

    /**
     * Sends expire and cache headers to the HTTP client
     * @param type $cacheTime The time in seconds to cache the content
     */
    public static function sendCacheHttpHeaders($cacheTime = null) {
        self::sendHttpHeaders(array_merge(
                        // Set the expires header
                        HeadersHelper::expires($cacheTime)
                        // Set the browser cache control for static files
                        , HeadersHelper::cache($cacheTime)
        ));
    }

    /**
     * Build expires headers for file
     *
     * @param int $seconds seconds before content expires
     *
     * @return array headers
     */
    public static function expires($seconds)
    {
        $headers = array();
        $headers[] = "Pragma: public";
        $headers[] = "Expires: " . date("r", time() + $seconds);

        return $headers;
    }
    
    /**
     * 
     * Build cache control header for file
     * 
     * @param string $filename
     * @param int $seconds how long the file should be cached in browser
     * @return array headers
     */
    public static function cache($seconds = null)
    {
    	$headers = array();
    	
    	$cache_string = "Cache-Control: public"; 
    	
    	if(isset($seconds) && is_numeric($seconds) && $seconds > 0) {
    		$cache_string = "Cache-Control: max-age=".$seconds.", public";
    	}
    	
    	$headers[] = $cache_string;
    	
    	return $headers;
    }
}
