<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget for view comments form for current model
 */
class CommentsFormWidget extends CommentsBaseWidget
{
    /**
     * @var string moderator email
     */
    public $moderatorEmail;

    /**
     * @var bool If notify moderator on new comments
     */
    public $notifyModeratorOnNew = true;

    /**
     * @var bool Allow anonymous comments or not
     */
    public $allowAnonymous = true;

    /**
     * @var What fields in anonymous form will be required (default all)
     */
    public $requiredForAnonymous = null;
    /**
     * @var bool If comments needs moderator approve
     */
    public $dontNeedApprove = false;

    /**
     * @var string Widget options hash, to get this options in controller
     */
    public $hash;

    /**
     * Make widgets options hash key and store options in session
     *
     * @return void
     */
    public function init()
    {
        $options = array(
            'contentId' => $this->contentId,
            'moderatorEmail' => $this->moderatorEmail,
            'notifyModeratorOnNew' => $this->notifyModeratorOnNew,
            'allowAnonymous' => $this->allowAnonymous,
            'requiredForAnonymous' => $this->requiredForAnonymous,
            'dontNeedApprove' => $this->dontNeedApprove
        );
        $this->hash = md5(serialize($options) . Yii::app()->getModule('comments')->hashSalt);
        if (Yii::app()->cache->get($this->hash) != $options) {
            Yii::app()->cache->set($this->hash, $options);
        }
        parent::init();
    }

    /**
     * Show comment form or login invitation based on widget options
     *
     * @return void
     */
    public function run()
    {
        if (!$this->allowAnonymous && Yii::app()->user->isGuest) {
            $this->render('CommentsFormWidget_Login');
        } else {
            $model = new Comment(Yii::app()->user->isGuest ? 'createAnonymous' : 'create');
            if(isset($this->requiredForAnonymous)){
                $model->setRequiredForAnonymous($this->requiredForAnonymous);
            }
            if (!Yii::app()->user->isGuest) {
                $model->email = Yii::app()->user->email;
                $profile = Profile::model()->findByPk(Yii::app()->user->id);
                $model->firstname = $profile->firstname;
                $model->lastname = $profile->lastname;
            }
            if (($modelData = Yii::app()->user->getState('Comment' . $this->contentId)) !== null) {
                $model->attributes = $modelData['attributes'];
                foreach ($modelData['errors'] as $attribute => $errors) {
                    foreach ($errors as $e) {
                        $model->addError($attribute, $e);
                    }
                }
                Yii::app()->user->setState('Comment' . $this->contentId, null);
            }
            $this->render('CommentsFormWidget', array('model' => $model));
        }
    }
}

?>