(function($){
	
	/**
	 * @class Library
	 */
	$.fg.Library = {
			
			/**
			 * Collection of library elements
			 * @type array
			 */
			elements: [],
		
		/**
		 * Initialize the library
		 * @return {void}
		 */
		init: function()
		{
			this.elements = [
				{
					type: "elements",
					title: t( "elements-categories.basic-elements" ),
					items: [
						{
							type: "space",
							title: t( "elements.space.title" )
						},
						{
							type: "dropdown",
							title: t( "elements.dropdown.title" )
						},
						{
							type: "radio",
							title: t( "elements.radio.title" )
						},
						{
							type: "checkbox",
							title: t( "elements.checkbox.title" )
						},
						{
							type: "captcha",
							title: t( "elements.captcha.title" )
						},
						/*{
							type: "text",
							title: t( "elements.text.title" )
						},*/
						{
							type: "input",
							title: t( "elements.input.title" )
						},
						{
							type: "textarea",
							title: t( "elements.textarea.title" )
						},
						{
							type: "next/previous",
							title: t( "elements.next/previous.title" )
						},
						{
							type: "reset",
							title: t( "elements.reset.title" )
						},
						{
							type: "submit",
							title: t( "elements.submit.title" )
						}
					]
				},
				{
					type: "elements",
					title: t( "elements-categories.combined-elements" ),
					items: []
				},
				{
					type: "elements",
					title: t( "elements-categories.special-elements" ),
					items: [
						/*{
							type: "mandatory_group",
							title: t( "title", "elements.mandatory_group" ),
							info: t( "info", "elements.mandatory_group" )
						},*/
						/*{
							type: "matrix",
							title: t( "elements.matrix.title" )
						}*/
					]
				},
				{
					type: "templates",
					name: "templates-global",
					title: t( "elements-categories.templates" ),
					items: []
				},
				{
					type: "templates",
					name: "templates-user",
					title: t( "elements-categories.mytemplates" ),
					items: []
				}
			];
			
			$( "#library" ).text( "" );
			$( "#library" ).append( this.render() );
			this.loadLibrary();
			$( "#library .accordion" ).accordion({
				autoHeight: false,
				active: $.fg.Editor.getUserState( "library-accordion")*1 || 0,
				change: function(event, ui)
				{
					var index = $(this).find("h3").index ( ui.newHeader[0] );
					$.fg.Editor.setUserState( "library-accordion", index );
				}
			});
			$( "#library ul.elements li a.library-element" )
				.click(function(e){
					e.preventDefault();
				})
				.draggable({
					helper: "clone",
					opacity: 0.7,
					containment: "body",
					appendTo: "#library .accordion",
					cancel: null
					//connectToSortable: $.fg.Elements.getFromStorage(0).object
				});
			$( "#library" ).surfacing();
		},
		
		/**
		 * Load library data from database
		 * @return {void}
		 */
		loadLibrary: function()
		{
			$.fg.Editor.callServerAction( "loadLibrary", {
				success: $.proxy( function(data) {

					if(data.templates.global.length) {
						this.setTabContent( "templates-global", this.renderTemplatesList( data.templates.global ) );
					}
					if(data.templates.user.length) {
						this.setTabContent( "templates-user", this.renderTemplatesList( data.templates.user ) );
					}
					
				}, this )
			} );
		},
		
		/**
		 * Render library area
		 * @return {object} library DOM node
		 */
		render: function()
		{
			var dF = $( "<div class=\"accordion\"></div>" );
			$.each(this.elements, function(k, v){
				var idPrefix = "library-tab-" + (typeof v.name === "undefined"? k : v.name);
				dF.append(
					$( "<h3></h3>", {
						id: idPrefix+"-title"
					} )
						.append(
							$( "<a></a>" )
								.attr( "href", "#" )
								.text( v.title )
					)
				);
				var content = $( "<div></div>", {
					id: idPrefix+"-content"
				} );
				switch(v.type)
				{
					case "elements":
						if(v.items.length) {
							var ul = $( "<ul class=\"list elements\"></ul>" );
							$.each(v.items, function(k, v) {
								var element = $.fg.Elements.create( v.type, false );
								var item = $( "<a></a>" )
									.attr( "href", "#" )
									.addClass( "library-item library-element" )
									.addClass( "element-"+v.type )
									.attr( "element-type", v.type );
								var libraryView = element
									? element.renderLibraryView()
									: null;
								if(libraryView) {
									item.append( libraryView )
								} else {
									item.text( v.title )
								}
								ul.append(
									$( "<li></li>" )
										.append( item )
								);
							});
							content.append( ul );
							break;
						}
					default:
						content.append( $("<div>", {text: t( "elements-library.no-items" )}).addClass( "empty" ) );
						break;
						
				}
				dF.append( content );
			});
			return dF;
		},
		
		/**
		 * Render lists of templates
		 * @param {array} array of templates
		 * @return {object} list DOM node
		 */
		renderTemplatesList: function(items)
		{
			var dF = document.createDocumentFragment();
			if(items.length) {
				var ul = $( "<ul class=\"list templates\"></ul>" );
				$.each(items, function(k, v) {
					var item = $( "<a></a>" )
						.attr( "href", "#" )
						.addClass( "library-item library-template" )
						.addClass( "id-"+v.id )
						.attr( "template-id", v.id );
					item.text( v.name );
					ul.append(
						$( "<li></li>" )
							.append( item )
					);
				});
				dF.append( ul );
			}
			ul.find( "li a.library-item" ).click( this.templateClickHandler );
			
			return dF;
		},
		
		/**
		 * Set library tab title
		 * @param {string} name tab name
		 * @param {string} title tab title
		 * @return {void}
		 */
		setTabTitle: function(name, title)
		{
			$( "#library-tab-"+name+"-title a" ).text( title );
		},
		
		/**
		 * Set library tab content
		 * @param {string} name tab name
		 * @param {mixed} content tab content
		 * @return {void}
		 */
		setTabContent: function(name, content)
		{
			$( "#library-tab-"+name+"-content" ).html( content );
		},
		
		/**
		 * Show library area
		 * @return {void}
		 */
		show: function()
		{
			if($( "#library" ).is( ":hidden" )) {
				$( "#library" ).show( "slide", {direction: "right"}, 200 );
			}
		},
		
		/**
		 * Hide library area
		 * @return {void}
		 */
		hide: function()
		{
			if($( "#library" ).is( ":visible" )) {
				$( "#library" ).hide( "slide", {direction: "right"}, 200 );
			}
		},
		
		/**
		 * Toggle (show / hide) library area
		 * @return {void}
		 */
		toggle: function()
		{
			$( "#library" ).toggle( "slide", {direction: "right"}, 200 );
		},
		
		/**
		 * Handler of click on template item<br />
		 * Loads and renders form from template
		 * @param {event} e
		 * @return {void}
		 */
		templateClickHandler: function(e)
		{
			e.preventDefault();
			$.fg.Dialog.confirm( {
				text: $.String.sub( t( "load_template_confirm" ), {name: $(this).text()} ),
				ok: $.proxy( function()
				{
					$.fg.Editor.callServerAction( "loadTemplate", {
						data: {id: $(this).attr( "template-id" )},
						success: function(data)
						{
							$.fg.Editor.loadFromJSON( data.json );
							$.fg.Notifier.add( $.String.sub( t( "load_template_success" ), {name: data.name} ), "success" )
						}
					} );
				}, this )
			} );
		}
		
	};
	
})(jQuery);
