<?php

/**
 * This widget allows you to transform an image and render the URL
 * to the new image in one step.
 **/
class GenticsImageStoreWidget extends BaseWidget
{
    /**
     * @var int Image ID in the content repository (without the type)
     */
    public $id            = null;

    /**
     * @var string  Publish path, relative to the Node.
     *              Note: This is only uset to prevent repository
     *              lookups (optimization) and for pretty cache file names
     */
    public $publishPath   = null;

    /**
     * @var string  Path or URL to an image (See PHP wrappers documentation)
     */
    public $filePath      = null;

    /**
     * @var array  Options for cropping the image
     */
    public $cropOptions   = array();

    /**
     * @var array  Options for resizing the image
     */
    public $resizeOptions = array();



    /**
     * Override base init. For skipping scripts and css init.
     * @return void
     */
    public function init()
    {
    }

    /**
     * Runs this widget
     */
    public function run()
    {
        $this->render('GenticsImageStoreWidget',
                array('url' => $this->transformImage()));
    }

    /**
     * Invokes the image transformation and returns the URL
     * @return string The URL to the transformed image
     */
    public function transformImage()
    {
        $genticsImageStoreModule = Yii::app()->getModule('genticsimagestore');

        $transformedImagePath = null;
        if (empty($this->id)) {
            // Try to load from filesystem / URL
            if (empty($this->filePath)) {
                throw new Exception("filePath or id must be set");
            }

            $transformedImagePath = $genticsImageStoreModule->transformImageByFileUrl(
               $this->filePath, $this->resizeOptions, $this->cropOptions);

            if ($transformedImagePath === null) {
                throw new Exception("Could not transform the image \""
                        . $this->filePath . "\"");
            }
        } else {
            // If a repository ID was specified, try to load the image from
            // the content repository
            if (empty($this->publishPath)) {
                $this->publishPath = null;
            }

            $transformedImagePath = $genticsImageStoreModule->transformImageByRepositoryId(
               $this->id, $this->resizeOptions, $this->cropOptions, $this->publishPath);

            if ($transformedImagePath === null) {
                throw new Exception("Could not transform the image \""
                        . "10008." . $this->id . "\" from repository");
            }
        }

        return $transformedImagePath;
    }
}
