<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class Googlesitemaps extends CComponent
{

    /**
     * @var string $rootfolderId contains the ID of the rootfolder.
    */
    public $rootfolderId;

    /**
     * @var string $hostParam can be set to build absolute Urls (eg 'http://www.gentics.com')
     */
    public $hostParam;

    /**
     * @var array $contentTypes contains the Type Id's which content to show (10007,10008,ect)
    */
    public $contentTypes;

    /**
     * @var string $hideParam contains the Name of the object Parameter which is can be set to not show content
    */
    public $hideParam;

    /**
     * @var string $containerParam contains the Name of the object Parameter which will be checked against the value to show the content
    */
    public $containerParam;

    /**
     * @var string $containerParamValue contains the value the object Parameter need to have to show the content
    */
    public $containerParamValue;

    /**
     * @var string $outputFormatValue sets the choosen Output Format in the View
    */
    public $outputFormatValue;

    /**
     * @var int the number of seconds in which the cached value will expire. 0 means never expire.
     */
    public $cacheTime;

    /**
     * @var string $gccRequestParams contains parameters that should be requested fron GCC
     */
    public $gccRequestParams;

    /**
     * @var string $lang contains current languagecode
     */
    public $lang;

    /**
     * array that contains categories tree
     * @var $data
     */

    public $data = array();

    /**
     * @var string $sorting contains sorting expression
     */
    public $sorting;


    /**
     * Construction that will load nodes tree for given $folderIds
     *
     * @param array  $folderIds          contains folder IDs of content to show
     *
     * @return \personalizedContent
     */
    public function __construct($rootfolderId = "", $hostParam = null, $contentTypes = null, $hideParam = null, $containerParam = null, $containerParamValue = null, $outputFormatValue = "", $cacheTime = null, $gccRequestParams = null, $sorting = null)
    {

        if ( $rootfolderId != "" ) {

            $this->lang = substr(Yii::app()->language, 0, 2);
            $this->hostParam = isset($hostParam)? $hostParam : Yii::app()->getModule('googlesitemaps')->hostParam;
            $this->contentTypes = isset($contentTypes)? $contentTypes : Yii::app()->getModule('googlesitemaps')->contentTypes;
            $this->hideParam = isset($hideParam)? $hideParam : Yii::app()->getModule('googlesitemaps')->hideParam;
            $this->containerParam = isset($containerParam)? $containerParam : Yii::app()->getModule('googlesitemaps')->containerParam;
            $this->containerParamValue = isset($containerParamValue)? $containerParamValue : Yii::app()->getModule('googlesitemaps')->containerParamValue;
            $this->cacheTime = isset($cacheTime)? $cacheTime : Yii::app()->getModule('googlesitemaps')->cacheTime;
            $this->gccRequestParams = isset($gccRequestParams)? $gccRequestParams : Yii::app()->getModule('googlesitemaps')->gccRequestParams;
            $this->sorting = isset($sorting)? $sorting : Yii::app()->getModule('googlesitemaps')->sorting;
            $this->outputFormatValue = $outputFormatValue;
            $this->hideParam = $hideParam;
            $this->rootfolderId = $rootfolderId;
            $this->_loadData();
            //$this->_cleanUp(Yii::app()->user->id);

        } else {
            return false;
        }
    }

    /**
     * Filter tree by some schema
     *
     * @param integer $user_id Id of the user to personalize tree for
     *
     * @return array
     */
    private function _cleanUp($user_id)
    {
        $maxTimestamp = $this->loadMaxUpdatetimestamp();
        $cacheId = 'googlesitemaps_PERS_'.md5($this->rootfolderId.implode(',', $this->contentTypes).$this->hideParam.$this->containerParam.$this->containerParamValue.$this->lang.$this->gccRequestParams.$this->sorting.$maxTimestamp);

        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            //get personalisation fields from settings
            $filters = $this->getFiltersList();
            $data = array();

            // apply filter here
            if ($filters && $this->data) {
                foreach ($this->data as $id=>$contentPage) {
                    if (!is_null($this->_unsetNodes($filters, $user_id, $contentPage))) {
                        $data[$id] = $contentPage;
                    }
                }
            }

            $this->data = $data;

            Yii::app()->cache->set($cacheId, $this->data);
        } else {
            $this->data = $data;
        }
    }

    /**
     * Private method that will unset not allowed data
     *
     * @param array $filters contains keys that will be used as filters
     * @param array $user_id contains user permissions
     * @param array $data    points to the array with nodes tree
     *
     * @return array|null
     */
    private function _unsetNodes($filters, $user_id, $data)
    {
        /*  check permissions for current node */
        $persAttributes = array();
        foreach ($filters as $filter) {
            if (!empty($data['attributes'][$filter])) {
                $persAttributes = array_merge($persAttributes, $data['attributes'][$filter]);
            }
        }

        //check if page has allowed startpageurl and exlude the branch if it does not.
        if ($data['contentid'] != in_array($data['contentid'], $this->folderIds)) {
            $excludeURLs = Yii::app()->getModule('googlesitemaps')->excludeURLs;
            if ((isset($data['attributes']['url']) && in_array($data['attributes']['url'], $excludeURLs)) || !isset($data['attributes']['url'])) {
                return null;
            }
        }

        return $data;
    }

    /**
     * Filter tree by some schema
     *
     * @return array|bool
     */
    public function getFiltersList()
    {
        if (Yii::app()->getModule('contentSource')->contentSource->usePersonalisation) {
            $filters = Yii::app()->getModule('contentSource')->contentSource->personalisationFields;
            return $filters;
        }

        return false;
    }

    /**
     * Load required tree data from cache. If not exists request Gentics Content Connector API for a new tree.
     *
     * @return array
     */
    private function _loadData()
    {
        $data = array();
        //$filtersList = '';
        $filter = '';

        $this->gccRequestParams = rtrim($this->gccRequestParams, ',');
        $requertFields = explode(',', $this->gccRequestParams);

        $requertFields[] = 'updatetimestamp';
        $requertFields[] = 'name';
        $requertFields[] = 'url';

        $requertFields = array_unique($requertFields);

        // get max timestamp for given folderIds
        //$maxTimestamp = $this->loadMaxUpdatetimestamp();
        $maxTimestamp = $this->_getYoungestTimestamp($this->rootfolderId, 'googlesitemaps_UTS_'.md5($this->rootfolderId.implode(',', $this->contentTypes).$this->hideParam.$this->containerParam.$this->containerParamValue.$this->cacheTime.$this->lang.$this->gccRequestParams));
  
        //create cache key that includes folderIds+language+gccRequestString+timestamp of last updated folder;
        $cacheId = 'googlesitemaps_'.md5($this->rootfolderId.implode(',', $this->contentTypes).$this->hideParam.$this->containerParam.$this->containerParamValue.$this->cacheTime.$this->lang.$this->gccRequestParams.$maxTimestamp);

        //Load data
        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            // Build additional Filter
            $filter = '(object.obj_type=="10002" OR object.obj_type=="'.implode('" OR object.obj_type=="', $this->contentTypes).'")';
            if ($this->hideParam != "") {
                $filter .= ' AND (object.'.$this->hideParam.' != "1")';
            }
            if ($this->containerParam != "" && $this->containerParamValue != "") {
                $filter .= ' AND (object.'.$this->containerParam.' == "'.$this->containerParamValue.'")';
            }
            $params = array(
                    'rootfilter' => 'object.contentid=="'.$this->rootfolderId.'"',
                    'childfilter' => $filter,
                    'sorting' => $this->sorting,
                    'navigation' => 'true',
                    'type' => 'php',
                    'filter' => ''
            );

            $params = Yii::app()->getComponent('repositoryApi')->addAdditionalParameters($params, 'contentRepositorySearch');

            $data = Yii::app()->repositoryApi->requestNavigation(
                $params,
                CMap::mergeArray($this->getFiltersList(), $requertFields),
                "notimeout"
            );
            $data = unserialize($data);

            if (isset($data['status']) && $data['status'] == 'ok') {
                unset($data['status']);
            } else {
                $data = array();
            }

            Yii::app()->cache->set($cacheId, $data);
        }
        $this->data = $data;
    }

    // Exchange getMaxUpdateTimestamp with call to API for "youngest"
    private function _getYoungestTimestamp($objectId, $objectCacheId)
    {
        $cacheId = $objectCacheId;
        $filter = '(object.obj_type=="10002" OR object.obj_type=="'.implode('" OR object.obj_type=="', $this->contentTypes).'")';
        if ($this->hideParam != "") {
            $filter .= ' AND (object.'.$this->hideParam.' != "1")';
        }
        if ($this->containerParam != "" && $this->containerParamValue != "") {
            $filter .= ' AND (object.'.$this->containerParam.' == "'.$this->containerParamValue.'")';
        }
        if (($updateTimestamp = Yii::app()->cache->get($cacheId)) === false) {
            $updateTimestamp = Yii::app()->repositoryApi->requestNavigation(
                array(
                    "rootfilter" => 'object.contentid=="' . $objectId . '"',
                    "type" => "youngest",
                    "sorting" => $this->sorting,
                    "childfilter" => $filter
                ),
                array()
            );
            Yii::app()->cache->set($cacheId, $updateTimestamp, $this->getLatestUpdateTimestampCacheTime());
        }
        return $updateTimestamp;
    }
    
    public function getLatestUpdateTimestampCacheTime()
    {
        return Yii::app()->getModule('navigation')->timestampCacheTime;
    }
}
