<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Widget for viewing navigation tree
 */

class NavigationWidget extends BaseJuiWidget
{
    /**
     * id of the root folder for current widget
     * @var startfolderId
     */
    public $startfolderId = 0;

    /**
     * id of the root folder for current widget
     * @var $activeElement
     */
    public $activeElement = 0;

    /**
     * says,that ALL leaves are opened
     * @var isSitemap
     */
    public $isSitemap = false;

    /**
     * defines is widget is in breadcrumb mode. If it is - then only elements from active path will be shown.
     * @var isBreadcrumb
     */
    public $isBreadcrumb = false;

    /**
     * this class will be applied to all List elements for current widget
     * @var cssClass
     */
    public $cssClass = 'navigation';

    /**
     * if maxLvl is set ans more then 0, then trre will be limited to this level
     * @var maxLvl
     */
    public $maxLvl = 0;

    /**
     * language
     * @var lang
     */
    public $lang = '';

    /**
     * Contains array with ids of opened folders
     * @var $activePath
     */
    public $activePath = array();

    /**
     * @var $hideFirstLevel bool defines if links of first level should be hidden
     */
    public $hideFirstLevel = false;

    /**
     * @var bool $usePersonalisation defines if personalisation should be performed
     */
    public $usePersonalisation = true;

    /**
     * @var string $sortby defines the field for sorting 
     */
    public $sortby = "sortorder:asc";
    /**
     * @var string comma separated extra request fields
     */
    public $extraRequestFields = '';


    /**
     * Initializes the widget.
     *
     * @return void
     */
    public function init()
    {
        //die('this is init');
    }

    /**
     * renders view
     *
     * @return void
     */
    public function run()
    {
        $tree = new NavigationTree($this->startfolderId, $this->activeElement, $this->sortby, $this->extraRequestFields);
        if ($tree->data) {
            $this->activePath = $tree->activePath;
            $this->lang = substr(Yii::app()->language, 0, 2);
            //$this->registerAssets();
            $this->render('NavigationWidget', array('tree'=>$tree));
        }
    }

    /**
     * Registers assets
     *
     * @return void
     */
    function registerAssets()
    {
        $cs = Yii::app()->getClientScript();
        $baseUrl = Yii::app()->getModule('navigation')->getAssetsUrl();

        //$cs->registerScriptFile($likeBaseUrl . '/js/navigation.js');
        $cs->registerCssFile($baseUrl . '/css/navigation.css');
    }

    /**
     * Registers assets
     *
     * @return string
     */
    function createCacheKey()
    {
        $key = 'startfolderId='.$this->startfolderId;
        $key .= 'activeElement='.$this->activeElement;
        $key .= 'isSitemap='.$this->isSitemap;
        $key .= 'cssClass='.$this->cssClass;
        $key .= 'maxLvl='.$this->maxLvl;
        $key .= 'lang='.$this->lang;
        $key .= 'hideFirstLevel='.$this->hideFirstLevel;
        $key .= 'usePersonalisation='.(int)$this->usePersonalisation;
        $key .= NavigationTree::getUpdatetimestampByContentId($this->startfolderId).serialize(PersonalisationAttribute::flatListForUser(intval(Yii::app()->user->id)));

        return md5($key);
    }
}

?>
